"""
This is the entry point for the command-line interface (CLI) application.

It can be used as a handy facility for running the task from a command line.

.. note::

    To learn more about Click visit the
    `project website <http://click.pocoo.org/5/>`_.  There is also a very
    helpful `tutorial video <https://www.youtube.com/watch?v=kNke39OZ2k0>`_.

    To learn more about running Luigi, visit the Luigi project's
    `Read-The-Docs <http://luigi.readthedocs.io/en/stable/>`_ page.

.. currentmodule:: databutton.cli
.. moduleauthor:: Databutton <support@databutton.com>
"""
# !/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import tarfile
import logging
import click
from .__init__ import __version__
import os
import webbrowser
import subprocess

LOGGING_LEVELS = {
    0: logging.NOTSET,
    1: logging.ERROR,
    2: logging.WARN,
    3: logging.INFO,
    4: logging.DEBUG,
}  #: a mapping of `verbose` option counts to logging levels


class Info(object):
    """An information object to pass data between CLI functions."""

    def __init__(self):  # Note: This object must have an empty constructor.
        """Create a new instance."""
        self.verbose: int = 0


# pass_info is a decorator for functions that pass 'Info' objects.
#: pylint: disable=invalid-name
pass_info = click.make_pass_decorator(Info, ensure=True)


# Change the options to below to suit the actual options for your task (or
# tasks).
@click.group()
@click.option("--verbose", "-v", count=True, help="Enable verbose output.")
@pass_info
def cli(info: Info, verbose: int):
    """Run databutton."""
    # Use the verbosity count to determine the logging level...
    if verbose > 0:
        logging.basicConfig(
            level=LOGGING_LEVELS[verbose]
            if verbose in LOGGING_LEVELS
            else logging.DEBUG
        )
        click.echo(
            click.style(
                f"Verbose logging is enabled. "
                f"(LEVEL={logging.getLogger().getEffectiveLevel()})",
                fg="yellow",
            )
        )
    info.verbose = verbose


@cli.command()
@pass_info
def hello(_: Info):
    """Say 'hello' to the nice people."""
    click.echo("databutton says 'hello'")


@cli.command()
def version():
    """Get the library version."""
    click.echo(click.style(f"{__version__}", bold=True))


@cli.command()
@click.option('--template', default='sales-forecasting', type=click.Choice(['sales-forecasting']), prompt='Select a template')
@click.option('--name', default='my-databutton-app', prompt='Name of the app')
def create(template, name):
    """Create an application with a template"""
    click.echo(click.style(f"You selected {template}"))

    # Fetch repository
    os.mkdir(name)
    res = requests.get(f"https://storage.googleapis.com/databutton-app-templates/{template}.tar.gz", stream=True)
    if res.ok:
        tmpname = f"/tmp/databutton-{template}.tar.gz"
        with open(tmpname, 'wb') as f:
            f.write(res.raw.read())
        tar_obj = tarfile.open(tmpname)
        tar_obj.extractall(name)
        click.echo()
        click.echo(click.style(f"Successfully created {name}", fg='green'))
        click.echo()
        click.echo(click.style(f"cd into the folder (cd {name}) and start hacking!"))
        click.echo(click.style("Use your favorite environment setup to install requirements with"))
        click.echo()
        click.echo(click.style("pip install -r requirements.txt", fg="green"))
        click.echo()
        click.echo(click.style("write `databutton dev` and make the data app yours!", fg="green"))
        click.echo()
        click.echo()

    else:
        click.echo(click.style(f"Error, could not get template {template}"))
        quit(1)


@cli.command()
def dev():
    """Run the development setup of databutton"""
    import uvicorn
    import sys
    from uvicorn.supervisors import ChangeReload
    dir_path = os.path.dirname(os.path.realpath(__file__))
    app_dir = os.path.join(dir_path, 'server')
    config = uvicorn.Config("server:app", reload=True, reload_excludes=".databutton/**/*.py")
    server = uvicorn.Server(config=config)
    server.force_exit = True
    sys.path.insert(0, app_dir)
    sock = config.bind_socket()
    supervisor = ChangeReload(config, target=server.run, sockets=[sock])
    supervisor.run()


@cli.command()
def login():
    """Login to databutton"""
    import uvicorn
    dir_path = os.path.dirname(os.path.realpath(__file__))
    app_dir = os.path.join(dir_path, 'auth')
    click.echo(click.style("Opening browser to authenticate"))
    click.echo(click.style("Please finish the login in the browser at http://localhost:8000"))
    click.echo(click.style("Exit this process with ctrl+c"))
    webbrowser.open('http://localhost:8000')
    uvicorn.run("server:app", app_dir=app_dir, reload=False, log_level='critical')


@cli.command()
def deploy():
    """Deploy to databutton"""
    click.echo(click.style("Deploying to project"))
    click.echo(click.style("Checking for dockerfile"))
    if (os.path.exists('Dockerfile')):
        print('found dockerfile')
    else:
        print('no dockerfile')
    subprocess.run('ngrok http ')
